from flask_restful import abort, request, Resource
from sqlalchemy.exc import DatabaseError

from vcinema.app import db
from vcinema.models.room import Room
from vcinema.dto.room import RoomDto


class RoomsApi(Resource):
    def get(self):
        try:
            rooms = Room.query.all()
        except DatabaseError as e:
            abort(500, status="error", message="An error occured whilst querying the database", exception=str(e))
        
        return [RoomDto.to_dict(room) for room in rooms], 200

    def post(self):
        req = request.get_json(force=True)
        name = req.get("name")
        source = req.get("source")

        if not name or not source:
            abort(400, status="error", message="Insufficient parameters specified")

        try:
            room = Room(name=name, source=source)
            db.session.add(room)
            db.session.commit()
        except DatabaseError as e:
            abort(500, status="error", message="An error occured whilst writing to the database", exception=str(e))

        return RoomDto.to_dict(room), 201


class RoomApi(Resource):
    def get(self, id):
        try:
            id = int(id)
        except ValueError:
            abort(400, status="error", message="ID must be an integer")

        try:
            room = Room.query.filter_by(id=id).first()
        except DatabaseError as e:
            abort(500, status="error", message="An error occured whilst querying the database", exception=str(e))

        if not room:
            abort(404, status="error", message=f"Room with ID {id} not found")

        return RoomDto.to_dict(room), 200

    def delete(self, id):
        try:
            id = int(id)
        except ValueError:
            abort(400, status="error", message="ID must be an integer")

        try:
            room = Room.query.filter_by(id=id).first()
        except DatabaseError as e:
            abort(500, status="error", message="An error occured whilst querying the database", exception=str(e))

        if not room:
            abort(404, status="error", message=f"Room with ID {id} not found")

        try:
            db.session.delete(room)
            db.session.commit()
        except DatabaseError as e:
            abort(500, status="error", message="An error occured whilst deleting a record from the database", exception=str(e))

        return "", 204