from uuid import uuid4

from flask_restful import abort, request, Resource
from sqlalchemy.exc import DatabaseError

from vcinema.app import db
from vcinema.models.room import Room
from vcinema.models.user import User
from vcinema.dto.user import UserDto


class UsersApi(Resource):
    def post(self):
        req = request.get_json(force=True)
        name = req.get("name")
        room_id = req.get("room-id")

        if not name or not room_id:
            abort(400, status="error", message="Insufficient parameters specified")

        try:
            room_id = int(room_id)
        except ValueError:
            abort(400, status="error", message="Room ID must be an integer")

        try:
            room = Room.query.filter_by(id=room_id).first()
        except DatabaseError as e:
            abort(500, status="error", message="An error occured whilst querying the database", exception=str(e))

        if not room:
            abort(400, status="error", message=f"Room with ID {room_id} not found")

        try:
            user = User(name=name, token=str(uuid4()), room=room)
            db.session.add(user)
            db.session.commit()
        except DatabaseError as e:
            abort(500, status="error", message="An error occured whilst writing to the database", exception=str(e))

        return UserDto.to_dict(user), 201